package com.simon.exception;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import com.simon.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Map;

/**
 * 序列化
 *
 * @author simon
 * @create 2018-05-31 18:16
 **/

public class CustomOauthExceptionSerializer extends StdSerializer<CustomOauthException> {
    private static Logger logger = LoggerFactory.getLogger(CustomOauthExceptionSerializer.class);

    public CustomOauthExceptionSerializer() {
        super(CustomOauthException.class);
    }

    @Override
    public void serialize(CustomOauthException value, JsonGenerator gen, SerializerProvider provider) throws IOException {
        logger.warn("CustomOauthException = " + value);

        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();

        gen.writeStartObject();
        gen.writeNumberField(AppConfig.CODE, value.getHttpErrorCode());
        gen.writeStringField(AppConfig.MESSAGE, value.getMessage());
        gen.writeNullField(AppConfig.DATA);
//        gen.writeStringField("message", "用户名或密码错误");
        //gen.writeStringField("path", request.getServletPath());
        //gen.writeStringField("timestamp", String.valueOf(new Date().getTime()));
        if (value.getAdditionalInformation()!=null) {
            for (Map.Entry<String, String> entry : value.getAdditionalInformation().entrySet()) {
                String key = entry.getKey();
                String add = entry.getValue();
                logger.warn("key = " + key);
                logger.warn("add = " + add);
                gen.writeStringField(key, add);
            }
        }
        gen.writeEndObject();
    }
}
